=== modified file '__init__.py'
--- __init__.py	2006-03-22 14:08:58 +0000
+++ __init__.py	2006-05-30 17:06:19 +0000
@@ -7,6 +7,7 @@
 from patchsource import BzrPatchSource
 from shelf import Shelf
 from bzrlib import DEFAULT_IGNORE
+from bzrlib.help import command_usage
 
 DEFAULT_IGNORE.append('./.shelf')
 DEFAULT_IGNORE.append('./.bzr-shelf*')
@@ -51,50 +52,88 @@
         s.shelve(source, all, message)
         return 0
 
+
+# The following classes are only used as subcommands for 'shelf', they're
+# not to be registered directly with bzr.
+
+class cmd_shelf_list(bzrlib.commands.Command):
+    """List the patches on the current shelf."""
+    aliases = ['list', 'ls']
+    def run(self):
+        self.shelf.list()
+
+
+class cmd_shelf_delete(bzrlib.commands.Command):
+    """Delete the patch from the current shelf."""
+    aliases = ['delete', 'del']
+    takes_args = ['patch']
+    def run(self, patch):
+        self.shelf.delete(patch)
+
+
+class cmd_shelf_switch(bzrlib.commands.Command):
+    """Switch to the other shelf, create it if necessary."""
+    aliases = ['switch']
+    takes_args = ['other-shelf']
+    def run(self, other_shelf):
+        s = Shelf(self.shelf.base, other_shelf)
+        s.make_default()
+
+
+class cmd_shelf_show(bzrlib.commands.Command):
+    """Show the contents of the specified patch."""
+    aliases = ['show', 'cat', 'display']
+    takes_args = ['patch']
+    def run(self, patch):
+        self.shelf.display(patch)
+
+
+class cmd_shelf_upgrade(bzrlib.commands.Command):
+    """Upgrade old format shelves."""
+    aliases = ['upgrade']
+    def run(self):
+        self.shelf.upgrade()
+
+
 class cmd_shelf(bzrlib.commands.Command):
-    """Perform various operations on your shelved patches. See also shelve.
-
-    Subcommands:
-        list   (ls)           List the patches on the current shelf.
-        delete (del) <patch>  Delete a patch from the current shelf.
-        switch       <shelf>  Switch to the named shelf, create it if necessary.
-        show         <patch>  Show the contents of the specified patch.
-        upgrade               Upgrade old format shelves.
-    """
+    """Perform various operations on your shelved patches. See also shelve."""
     takes_args = ['subcommand', 'args*']
 
+    subcommands = [cmd_shelf_list, cmd_shelf_delete, cmd_shelf_switch,
+        cmd_shelf_show, cmd_shelf_upgrade]
+
     def run(self, subcommand, args_list):
         import sys
 
+        cmd = self._get_cmd_object(subcommand)
         source = BzrPatchSource()
         s = Shelf(source.base)
-
-        if subcommand == 'ls' or subcommand == 'list':
-            self.__check_no_args(args_list, "shelf list takes no arguments!")
-            s.list()
-        elif subcommand == 'delete' or subcommand == 'del':
-            self.__check_one_arg(args_list, "shelf delete takes one argument!")
-            s.delete(args_list[0])
-        elif subcommand == 'switch':
-            self.__check_one_arg(args_list, "shelf switch takes one argument!")
-            s = Shelf(source.base, args_list[0])
-            s.make_default()
-        elif subcommand == 'show':
-            self.__check_one_arg(args_list, "shelf show takes one argument!")
-            s.display(args_list[0])
-        elif subcommand == 'upgrade':
-            self.__check_no_args(args_list, "shelf upgrade takes no arguments!")
-            s.upgrade()
-        else:
-            raise CommandError("Unknown shelf subcommand '%s'" % subcommand)
-
-    def __check_one_arg(self, args, msg):
-        if args is None or len(args) != 1:
-            raise CommandError(msg)
-
-    def __check_no_args(self, args, msg):
-        if args is not None:
-            raise CommandError(msg)
+        cmd.shelf = s
+        return cmd.run_argv_aliases(args_list)
+
+    def _get_cmd_object(self, cmd_name):
+        for cmd_class in self.subcommands:
+            for alias in cmd_class.aliases:
+                if alias == cmd_name:
+                    return cmd_class()
+        raise CommandError("Unknown shelf subcommand '%s'" % cmd_name)
+
+    def help(self):
+        lines = ["%s\n\nSubcommands:" % self.__doc__]
+
+        sub_help = []
+        maxlen = 0
+        for cmd_class in self.subcommands:
+            usage = command_usage(cmd_class())
+            usage = usage.replace('bzr shelf-', '')
+            sub_help.append((usage, cmd_class.__doc__))
+            maxlen = max(maxlen, len(usage))
+
+        for usage, doc in sub_help:
+            lines.append("  %-*s  %s" % (maxlen, usage, doc))
+
+        return '\n'.join(lines)
+
 
 class cmd_unshelve(bzrlib.commands.Command):
     """Restore the most recently shelved changes to the current tree.

